
package sam.cipher;


/** Implementation of the <a href="mailto:lars.knudsen@ii.uib.no">Deal</a> algorithm
 **
 ** Retrofitted for personal use by <a href=mailto:sam@glacote.com>Guillaume Lacte</a>.
 **
 ** DEAL is a 128-bit, variable key-size (128-, 192- and 256-bit) symmetric
 ** cipher.<p>
 **
 ** DEAL was designed by <a href="mailto:lars.knudsen@ii.uib.no">Lars Ramkilde Knudsen</a>
 ** and is described by the designer in: DEAL -- A 128-bit Block
 ** Cipher, February 21, 1998 (postscript file).<p>
 **
 ** <b>Copyright</b> &copy; 1997, 1998
 ** <a href="http://www.systemics.com/">Systemics Ltd</a> on behalf of the
 ** <a href="http://www.systemics.com/docs/cryptix/">Cryptix Development Team</a>.
 ** <br>All rights reserved.<p>
 **
 ** <b>Disclaimer</b>: this implementation is currently buggy and this cipher is thus
 ** <b>not</b> registered by default. I believe this is due to <em>my</em> modifications.
 **
 ** @author  Raif S. Naffah
 **
 ** Copyright (c) 1997, 1998 Systemics Ltd on behalf of
 ** the Cryptix Development Team. All rights reserved.
 **/
public class Deal extends Cipher {

		static {
			if (false)  // Currently buggy ...
			Cipher.REGISTRY.register( new ReflexiveFactory(Deal.class, "Deal", "Deal algorithm") {
				protected int getPreferredKeySize(int size) { return highestIn(size, new int[] { 128, 192, 256 } ); }
			}	);
		}

		public Deal(sam.cipher.Key key) { super(key);	}

		/** The DES preferred size is always <CODE>16</CODE> **/
		public int getPreferredSize(int size) { return 16; }

		/** Key implementation that is adapted to the {@link Deal} cipher. **/
    public static class Key extends sam.cipher.Key {
			/** The actual key material **/
			final transient DES[] des;

				/** Mandatory constructor implementation.
				 ** @see sam.cipher.ReflexiveFactory
				 **/
			public Key(long[] bits) {
				this(makeKey( new sam.cipher.Key(bits, 64, true).getBytes() ), true);
			}

			private Key(DES[] des, boolean direction) {
				super(new long[0], 0, direction);
				this.des = des;
			}

			/** Wether this key is empty **/
			public boolean isEmpty() { return (des.length == 0);	}

			/** Inverses this key **/
			public sam.cipher.Key inverse() {
				return new Key(des, ! direction);
			}

    }

    /** Encrypts exactly one block of plaintext.
     ** @param in          The plaintext.
     ** @param inOffset    Index of in from which to start considering data.
     ** @param key  The session key to use for encryption.
     **/
    protected void encrypt(byte[] in, int inOffset, sam.cipher.Key key) {
			byte[] encrypted = blockEncrypt(in, inOffset, ((Key) key).des );
			System.arraycopy(encrypted, 0, in, inOffset, encrypted.length);
		}

    /** Decrypts exactly one block of plaintext.
     ** @param in          The plaintext.
     ** @param inOffset    Index of in from which to start considering data.
     ** @param key  The session key to use for encryption.
     **/
    protected void decrypt(byte[] in, int inOffset, sam.cipher.Key key) {
			byte[] decrypted = blockEncrypt(in, inOffset, ((Key) key).des );
			System.arraycopy(decrypted, 0, in, inOffset, decrypted.length);
		}


// Basic API methods
//...........................................................................

    /**
     * Expand a user-supplied key material into a session key.
     *
     * @param key  The 128/192/256-bit user-key to use.
     * @exception  InvalidKeyException  If the key is invalid.
		 * @return int[][]
     */
    private static DES[] makeKey (byte[] k) {
        int s = k.length / 8;
        if (!(s == 2 || s == 3 || s == 4))
					return null;
        //
        // format user key material into K[] DES key material objects
        // each being 64-bit wide; ie. an array of 8 bytes
        //
        byte[][] K = new byte[s][8];
        int i, j, offset = 0;
        for (i = 0; i < s; i++)
					for (j = 0; j < 8; j++)
             K[i][j] = k[offset++];
        //
        // out of these s DES keys make r new DES keys RK[]; r being
        // the appropriate number of rounds for the given user key
        // material supplied
        //
        long[] RK = null;
        // the formulae for computing RK[] use a fixed DES key, we'll
        // note it as Kf
				DES des = new DES( new DES.Key(0x0123456789ABCDEFL) );
        // three cases are possible depending on the value of s
        switch (s) {
        case 2:
					RK = new long[6];
					RK[0] = desEncrypt(des,         K[0]             );
					RK[1] = desEncrypt(des, xor(    K[1],     RK[0]) );
					RK[2] = desEncrypt(des, xor(xor(K[0], 1), RK[1]) );
					RK[3] = desEncrypt(des, xor(xor(K[1], 2), RK[2]) );
					RK[4] = desEncrypt(des, xor(xor(K[0], 3), RK[3]) );
					RK[5] = desEncrypt(des, xor(xor(K[1], 4), RK[4]) );
					break;
        case 3:
					RK = new long[6];
					RK[0] = desEncrypt(des,         K[0]             );
					RK[1] = desEncrypt(des, xor(    K[1],     RK[0]) );
					RK[2] = desEncrypt(des, xor(    K[2],     RK[1]) );
					RK[3] = desEncrypt(des, xor(xor(K[0], 1), RK[2]) );
					RK[4] = desEncrypt(des, xor(xor(K[1], 2), RK[3]) );
					RK[5] = desEncrypt(des, xor(xor(K[2], 3), RK[4]) );
					break;
        case 4:
					RK = new long[8];
					RK[0] = desEncrypt(des,         K[0]             );
					RK[1] = desEncrypt(des, xor(    K[1],     RK[0]) );
					RK[2] = desEncrypt(des, xor(    K[2],     RK[1]) );
					RK[3] = desEncrypt(des, xor(    K[3],     RK[2]) );
					RK[4] = desEncrypt(des, xor(xor(K[0], 1), RK[3]) );
					RK[5] = desEncrypt(des, xor(xor(K[1], 2), RK[4]) );
					RK[6] = desEncrypt(des, xor(xor(K[2], 3), RK[5]) );
					RK[7] = desEncrypt(des, xor(xor(K[3], 4), RK[6]) );
					break;
        }
        // DEAL session keys become then the set of the r session keys built
        // using the RK[]
        DES[] sessionKey = new DES[RK.length];
        for (i = 0; i < RK.length; i++)
					sessionKey[i] = new DES( new DES.Key(RK[i]) );
        return sessionKey;
    }

		private static long desEncrypt(DES des, byte[] data) {
			byte[] encrypted = (byte[]) data.clone();
			des.cipher(encrypted);
			return unstoreLong(encrypted, 0);
		}


    /**
     * Encrypt exactly one block of plaintext.
     *
     * @param  in         The plaintext.
     * @param  inOffset   Index of in from which to start considering data.
     * @param  sessionKey The session key to use for encryption.
     * @return The ciphertext generated from a plaintext using the session key.
     */
    private static byte[] blockEncrypt (byte[] in, int inOffset, DES[] rKey) {
        int rounds = rKey.length;

        byte[] L0 = new byte[8];       // divide input into 2 64-bit entities
        byte[] R0 = new byte[8];
        byte[] L1, R1; // temp arrays
        System.arraycopy(in, inOffset,     L0, 0, 8);
        System.arraycopy(in, inOffset + 8, R0, 0, 8);

        DES.IP(L0);                   // apply DES IP to both sides
        DES.IP(R0);

        R1 = (byte[]) L0.clone();               // j = 1
        rKey[0].cipher(L0);
        L1 = xor(L0, R0);
        R0 = (byte[]) L1.clone();               // j = 2
        rKey[1].cipher(L1);
        L0 = xor(L1, R1);
        R1 = (byte[]) L0.clone();               // j = 3
        rKey[2].cipher(L0);
        L1 = xor(L0, R0);
        R0 = (byte[]) L1.clone();               // j = 4
        rKey[3].cipher(L0);
        L0 = xor(L1, R1);
        R1 = (byte[]) L0.clone();               // j = 5
        rKey[4].cipher(L0);
        L1 = xor(L0, R0);
        R0 = (byte[]) L1.clone();               // j = 6
				rKey[5].cipher(L0);
        L0 = xor(L1, R1);
        if (rounds == 8) {
					R1 = (byte[]) L0.clone();           // j = 7
					rKey[6].cipher(L0);
					L1 = xor(L0, R0);
					R0 = (byte[]) L1.clone();           // j = 8
					rKey[7].cipher(L0);
					L0 = xor(L1, R1);
        }
        DES.FP(L0);                   // apply DES FP to both sides
        DES.FP(R0);
        byte[] result = new byte[16];
        System.arraycopy(L0, 0, result, 0, 8);
        System.arraycopy(R0, 0, result, 8, 8);
        return result;
    }

    /**
     * Decrypt exactly one block of ciphertext.
     *
     * @param  in         The ciphertext.
     * @param  inOffset   Index of in from which to start considering data.
     * @param  sessionKey The session key to use for decryption.
     * @return The plaintext generated from a ciphertext using the session key.
     */
    private static byte[] blockDecrypt (byte[] in, int inOffset, DES[] rKey) {
        int rounds = rKey.length;

        byte[] L1 = new byte[8];       // divide input into 2 64-bit entities
        byte[] R1 = new byte[8];
        byte[] L0, R0; // temp arrays
        System.arraycopy(in, inOffset,     L1, 0, 8);
        System.arraycopy(in, inOffset + 8, R1, 0, 8);
        DES.IP(L1);                   // apply DES IP to both sides
        DES.IP(R1);
        if (rounds == 8) {
            L0 = (byte[]) R1.clone();           // j = 8
            rKey[7].cipher(R1);
            R0 = xor(L1, R1);
            L1 = (byte[]) R0.clone();           // j = 7
            rKey[6].cipher(R1);
            R1 = xor(L0, R0);
        }
        L0 = (byte[]) R1.clone();               // j = 6
				rKey[5].cipher(R1);
        R0 = xor(L1, R1);
        L1 = (byte[]) R0.clone();               // j = 5
				rKey[4].cipher(R1);
        R1 = xor(L0, R0);
        L0 = (byte[]) R1.clone();               // j = 4
				rKey[3].cipher(R1);
        R0 = xor(L1, R1);
        L1 = (byte[]) R0.clone();               // j = 3
				rKey[2].cipher(R1);
        R1 = xor(L0, R0);
        L0 = (byte[]) R1.clone();               // j = 2
				rKey[1].cipher(R1);
        R0 = xor(L1, R1);
        L1 = (byte[]) R0.clone();               // j = 1
				rKey[0].cipher(R1);
        R1 = xor(L0, R0);

        DES.FP(L1);                   // apply DES FP to both sides
        DES.FP(R1);
        byte[] result = new byte[16];
        System.arraycopy(L1, 0, result, 0, 8);
        System.arraycopy(R1, 0, result, 8, 8);
        return result;
    }

// own methods
//...........................................................................

    /** @return A 64-bit entity = x ^ y. */
		private static byte[] xor(byte[] a, long y) {
			byte[] y_ = new byte[8];
			store(y_, 0, y);
			return xor(a, y_);
		}

    /** @return A 64-bit entity = x ^ y. */
    private static byte[] xor (byte[] x, byte[] y) {
        byte[] result = (byte[]) x.clone();
        for (int i = 0; i < 8; i++)
            result[i] ^= y[i];
        return result;
    }

    /**
     * @param x  A DES key as a byte[].
     * @param y  A bit number/position in a 64-bit ordinal string where the
     *      <i>y</i><sup>th</sup> bit (and no other) is set. Allowed values
     *      for <i>y</i> are 1, 2, 3 and 4. The bits are numbered, as described
     *      in FIPS 46-2 (Data Encryption Standard --DES), from left to right,
     *      ie. the left most bit of the block is bit zero.
     * @return  A 64-bit entity result of xoring both input values together.
     */
    private static byte[] xor (byte[] x, int y) {
       byte[] result = (byte[]) x.clone();
       result[0] ^= (0x80 >>> (y-1)) & 0xFF;
       return result;
    }


}
