
package sam.cipher;

import java.util.*;



/** Central repository of all <CODE>Cipher</CODE> implementations
 **/
public class Registry {

	/** Dictionnary of all registered ciphers **/
  private final Map ciphers = new HashMap();

	/*
  public String register(String className, String name, String description) throws Error {
		try {
			return register( Class.forName(className), name, description );  // May also throw class loading errors
		} catch (ClassNotFoundException e) {
			throw new Error(e);
		}
	}
	*/

		/**
			Registers a new <CODE>Cipher</CODE> class.
	*/
	/*
  public synchronized String register(Class cipher, String name, String description) throws Error {
		return register( new ReflexiveFactory(cipher, name, description) );  // synchonized call
  }
	*/


   /** Registers a new <CODE>Cipher</CODE> class. **/
  public synchronized String register(Factory factory) throws Error {
		final String name = factory.name;
		if ((name == null) || ("".equals(name)))
				throw new Error("Attempt to register a cipher implementation with an empty registration name");
		ciphers.put(name, factory);  // synchronized call
		return name;
  }

	/** Wether specified name is that of an already registered <CODE>Cipher</CODE> factory **/
	public synchronized boolean exists(String name) { return ciphers.keySet().contains(name); }

  /** @return the number of registered ciphers. **/
	public synchronized int getNumberOfCiphers() { return ciphers.size(); }

  /** Builds a new instance of a registered cipher.
	 ** @param name the registration-name of the desired cipher (most often its class name), or a valid new class name.
	 ** @param keyBits the bits to fill the cipher's key with.
	 ** @param mode switch between encryption and decryption.
	 ** Note that the key shall be parsed through plain text and key mode; however this might be used for
	 ** benchmarking or debugging purpose.
	 **/
  Cipher newCipher(String name, long[] keyBits, boolean mode) throws Error {
		Object factory = ciphers.get(name);
		if (factory == null)
			throw new Error("Unregistered cipher: " + name);
		return ((Factory) factory).newInstance(keyBits, mode);
  }


	/** Builds a new <CODE>Cipher</CODE>.
	 ** Defaults key parsing is <CODE>Key.PLAIN</CODE>
	 ** @see Key.PLAIN
	 **/
	Cipher newCipher(String name, String plainKey, boolean mode) throws Error {
		return newCipher(name, Key.parse(plainKey, Key.PLAIN, 64), mode);
	}


  /** Gets the names of all registered ciphers. **/
  public String[] getCiphers() { return sam.toolkit.Text.getKeyStrings(ciphers); }

  /** Gets the description of the cipher registered under specified name,
	 ** or default string if not available (This never returns <CODE>null</CODE>).
	 ** @param name the registration name of desired cipher.
	 **/
  public String getDescription(String name) {
		Factory factory = (Factory) ciphers.get(name);
		return (factory == null) ? ("(Not registered: " + name) : factory.description;
  }

}

