
package sam.cipher;

/**
 * Basic XOR cipher.
 ** <p><b>This cipher is inherently unsafe</b>: given its key size <CODE>n</CODE>, well-known bytes of unciphered data
 ** might be found at position <CODE>j</CODE> modulo <CODE>n</CODE> for all <CODE>j</CODE> less than <CODE>n</CODE>
 ** with a dramatically high probability. Because the <CODE>XOR</CODE> function is its self-inverse the full key can
 ** then be obtained for the <b>full</b> data. This is especially true with non-binary files (like Microsoft Word or
 ** HTML documents).
 ** <p>Thus it is <b>not</b> intended to be used solely, but rather as a masking step in the ciphering stack.
 ** It is especially usefull in top of the huffman compression algorithm.
 **/
public class XOR extends Cipher {

		static {
			Cipher.REGISTRY.register( new ReflexiveFactory(XOR.class, "XOR", "Basic bit-wise exclusive-or filter") );
		}

		public XOR(Key key) {
			super(key);
		}

		/** Cyclic exclusive-or of the bytes of data. **/
		protected void encrypt(byte[] data, int offset, Key key) {
			final byte[] bytes = key.getBytes();
			for (int i=0; i<bytes.length; ++i)
				data[offset + i] ^= (byte) (bytes[i % bytes.length] & 0xff);
		}

		/** Same as <CODE>encrypt</CODE> since the XOR function is involutive	**/
		protected void decrypt(byte[] data, int offset, Key key) {	encrypt(data, offset, key); }

		protected int getPreferredSize(int size) {
			return key.getBytes().length;
		}

}

