
package sam.event;


import sam.entry.Entry;

import java.io.PrintWriter;


/** Event concerning file.
 **/
public class FileEvent {

	/** The <CODE>Entry</CODE> concerned by this event **/
	public final Entry entry;

	/** A message describing this event in a human-readable format **/
	public final String message;

	public FileEvent(Entry entry, String message) { this.entry = entry; this.message = message; }

	/** Gets the relative path of the file concerned by this event.
	 ** <b>Warning</b>: the directory separator is that of <CODE>Entry</CODE> i.e. the <CODE>"/"</CODE>
	 ** which might differ from the <CODE>java.io.File.separator</CODE> especially under <CODE>Win32</CODE>.
	 ** @see sam.entry.Entry.name
	 **/
	public String path() { return entry.name; }

	/** Gets a standardized representation of this event.
	 ** This is currently implemented as something like <CODE>"myDir/myDir/myFile: here is the message"</CODE>
	 **/
	public String toString() { return path() + ": " + message; }

	/** Prints this event onto specified <CODE>Writer</CODE>
	 ** @see #toString
	 **/
	public void printTo(PrintWriter output) {
		output.println( toString() );
		output.flush();
	}

}
