

package sam.event;


import java.io.*;


/** Trivial <CODE>FileEvent</CODE> listener that reports all file events to a <CODE>Writer</CODE>
 ** <p>Useful for easy inner-class subclassing.
 **/
public class FileEventWriter extends FileEventListener {
	private PrintWriter output;

	/** Builds a new listener that tracks back all events to standard output **/
	public FileEventWriter() { this( new PrintWriter(System.out) ); }

	/** Builds a new listener that tracks back all events to specified writer **/
	public FileEventWriter(Writer writer) { this( new PrintWriter(writer) ); }

	/** Builds a new listener that tracks back all events to specified output **/
	public FileEventWriter(PrintWriter output) { this.output = output; }

	/**	Called whenever a file is processed **/
	public void processed(FileEvent event) { output.println("Processed: " + event); }

	/**	Called whenever the handling of a file failed **/
	public void failed(FileEvent event) { output.println("Failed: " + event); }

	/**	Called whenever a file is missing **/
	public void missing(FileEvent event) { output.println("Missing: " + event); }

	/**	Called whenever a file is overwritten **/
	public void overwritten(FileEvent event) { output.println("Overwritten: " + event); }

	/**	Called whenever a file does not match its archived version **/
	public void mismatching(FileEvent event) { output.println("Mismatching: " + event); }

	/**	Called whenever a file is skipped,
	 ** for example because it is too old or does not match specified regular expression
	 **/
	public void skipped(FileEvent event) { output.println("Skipped: " + event); }

}
