
package sam.io;

import sam.ui.UserInterface;

import java.io.*;


/** <CODE>OutputStream</CODE> that interrupts <CODE>close()</CODE> calls.
 ** Usefull to avoid closing an archive while writing a single entry.
 **/
public class CloseAvoiderOutputStream extends java.io.OutputStream {
	private final java.io.OutputStream delegee;

	public CloseAvoiderOutputStream(java.io.OutputStream delegee) { this.delegee = delegee; }

	public void flush() throws IOException { delegee.flush(); }
	public void write(int b) throws IOException { delegee.write(b); }
	public void write(byte[] data, int begin, int end) throws IOException { delegee.write(data, begin, end); }

	/** Avois closing the underlying stream, but flushes it to mimic a <CODE>close()</CODE> call **/
	public void close() throws IOException {
		UserInterface.get().log().log(4, CloseAvoiderOutputStream.class, "Attempt to close stream, ignored");
		delegee.flush();
	}

	/** Really closes the stream **/
	public void reallyDoClose() throws IOException {
		UserInterface.get().log().log(1, CloseAvoiderOutputStream.class, "Really closing stream");
		delegee.close();
	}

}

