

package sam.io;


import java.io.*;



/** Classes for input/output on streams two bytes at a time
 **/

		public class DoubleByte {

			public static int firstByte(int doubleByte)  {	return (doubleByte >>> 8) & 0xff;	}
			public static int secondByte(int doubleByte) {  return doubleByte& 0xff;	}
			public static int mergeBytes(int a, int b)   {	return ((a & 0xff) << 8) | (b & 0xff);	}


			/** Exception thrown from <CODE>DoubleByte</CODE> classes **/
			public static class Exception extends java.io.IOException {
				public Exception(String message) { super(message); }
				public Exception() { super("DoubleByte exception"); }
			}
			/** Simple double-byte reader abstraction **/
			public static interface Reader { public int read() throws IOException; }
			/** Simple double-byte writer abstraction **/
			public static interface Writer { public void write(int doubleByte) throws IOException; }

			/** Writes double-bytes into standard single-byte <CODE>OutputStream</CODE> **/
			public static class OutputStreamWriter implements Writer {
				private java.io.OutputStream delegee;
				public OutputStreamWriter(java.io.OutputStream delegee) { this.delegee = delegee;	}
				public void write(int doubleByte) throws Exception {
						try {
							delegee.write( firstByte(doubleByte) );
							delegee.write( secondByte(doubleByte) );
						} catch (IOException e) {
							throw new Exception("Write error: " + sam.toolkit.Text.getMessage(e) );
						}
				}
			}

			/** Reads double-bytes from standard single-byte <CODE>InputStream</CODE> **/
			public static class InputStreamReader implements Reader {
				private java.io.InputStream delegee;
				public InputStreamReader(java.io.InputStream delegee) { this.delegee = delegee;	}
				private int delegeeRead() throws IOException { int a = delegee.read(); if (a == -1) throw new EOFException(); else return a; }
				public int read() throws IOException {
					int a;
					try {
						a = delegeeRead();
					} catch (EOFException e){
						throw e;
					} catch (IOException e) {
						throw new Exception("Read error: " + sam.toolkit.Text.getMessage(e) );
					}
					try {
						return mergeBytes(a, delegeeRead() );
					} catch (IOException e) {
						return mergeBytes(a, 0);
					}
				}
			}

			/** Reads single bytes from a double-bytes <CODE>Reader</CODE> **/
			public static class ReaderInputStream extends java.io.InputStream {
				protected Reader reader;
				private int value;
				private boolean isAtFirstByte;

				protected ReaderInputStream() {	this(null);	}

				public ReaderInputStream(Reader reader) {
					this.reader = reader;
					isAtFirstByte = true;  // <CODE>false</CODE> since <CODE>read()</CODE> inverts it first
				}

				public synchronized int read() throws IOException {
					if (isAtFirstByte) {  // Meaning was not
						value = reader.read();  // Which may throw an <CODE>Exception</CODE>
						if (value < 0)
							throw new EOFException();  // Could return -1 instead
						isAtFirstByte = false;
						return firstByte(value);
					} else {
						isAtFirstByte = true;
						return secondByte(value);
					}
				}

			}

			/** Writes single bytes into a double-bytes <CODE>Writer</CODE> **/
			public static class WriterOutputStream extends java.io.OutputStream {
				protected Writer writer;
				private int value;
				private boolean isAtFirstByte;

				protected WriterOutputStream() { this(null); }

				public WriterOutputStream(Writer writer) {
					this.writer = writer;
					isAtFirstByte = true;  // This time <CODE>write()</CODE> inverts it <b>after</b> the test
				}

				public synchronized void write(int b) throws IOException {
					if (isAtFirstByte)
						value = b & 0xff;
					else
						writer.write( mergeBytes(value, b) );  // Which may throw an <CODE>Exception</CODE>
					isAtFirstByte = ! isAtFirstByte;
				}

				public void flush() throws IOException {
					if ( ! isAtFirstByte)
						writer.write( value << 8 );
				}

				public void close() throws IOException { flush();	}

			}

		}


