

package sam.io;

import java.io.*;


public class Huffman16 extends Huffman {

	/**
	   Creates a new well-balanced Huffman tree.
	   Default granularity is 1.
	*/
	public Huffman16() { super(16); }

	/** Stream for decoding 16 bits huffman-encoded data
	 ** @param delegee the 16 bits stream to decode from
	 **/
	public InputStream input(InputStream delegee) {
		final LeafInputStream leafInputStream = new LeafInputStream(delegee);
		return new DoubleByte.ReaderInputStream( new DoubleByte.Reader() {
			public int read() throws IOException {
				final Leaf leaf = leafInputStream.read();
				if (leaf == null)
					throw new EOFException();
				return leaf.value;
				}
			public void finalize() {  try { leafInputStream.close(); } catch (Throwable t) {}  }
			}
    );
	}

	/** Stream for encoding 16 bits huffman-encoded data
	 ** @param delegee the 16 bits stream to encode to
	 **/
	public OutputStream output(OutputStream delegee) {
	  final LeafOutputStream leafOutputStream = new LeafOutputStream(delegee);
		return new DoubleByte.WriterOutputStream( new DoubleByte.Writer() {
			public void write(int doubleByte) throws IOException {	leafOutputStream.write( get(doubleByte) );	}
			public void finalize() {  try { leafOutputStream.close(); } catch (Throwable t) {}  }
		}
		);
	}

}



