

package sam.io;

import java.io.*;


public class Huffman8 extends Huffman {

	/**  Creates a new well-balanced Huffman tree. **/
	public Huffman8() {	super(8);	}

	/** Stream for decoding 8 bits huffman-encoded data
	 ** @param delegee the 8 bits stream to decode from
	 **/
	public InputStream input(InputStream delegee) {
		final LeafInputStream leafInputStream = new LeafInputStream(delegee);
		return new InputStream() {
			public int read() throws IOException {
			  final Leaf leaf = leafInputStream.read();
				if (leaf == null)
				  throw new EOFException();
				return leaf.value;
			}
			public void close() throws IOException {  leafInputStream.close(); }
		};
	}


	/** Stream for encoding 8 bits huffman-encoded data
	 ** @param delegee the 8 bits stream to encode to
	 **/
	public OutputStream output(OutputStream delegee) {
		final LeafOutputStream leafOutputStream = new LeafOutputStream(delegee);
		return new OutputStream() {
			public synchronized void write(int b) throws IOException {	leafOutputStream.write( get(b) );	 }
			public void flush() throws IOException { leafOutputStream.flush(); }
			public void close() throws IOException { leafOutputStream.close(); }
		};
	}


}


