

package sam.io;


import sam.toolkit.Workshop;


/** Class for filtering <CODE>byte[]</CODE> arrays
 ** using multiple threads in parallel to increase performance.
 **/
public class MultiThreadedByteArrayFilter extends ByteArrayFilter {
	private final ByteArrayFilter delegee;
	private Workshop workshop;
	private int minimalBlockSize;

	public MultiThreadedByteArrayFilter(ByteArrayFilter filter, Workshop workshop, int minimalBlockSize) {
		super(filter.length);
		this.delegee = filter;
		this.workshop = workshop;
		this.minimalBlockSize = minimalBlockSize;
	}

	protected void filter(byte[] data, int begin) { delegee.filter(data, begin); }

	public void filter(final byte[] data, int begin, int length) {
		int numberOfRequiredWorkers = workshop.numberOfWorkers();
		int subBlockLength = length / numberOfRequiredWorkers;
		if (subBlockLength < minimalBlockSize) {
			subBlockLength = minimalBlockSize;
			numberOfRequiredWorkers = (int) Math.ceil( (float)length / (float)subBlockLength );
		}
		for (int offset = begin; offset < numberOfRequiredWorkers; offset += subBlockLength) {
			final int _offset = offset,  // For anonymous inner class
					_subBlockLength = ((offset + subBlockLength - begin) <= length) ? subBlockLength : (length - (offset - begin) );
			workshop.startAsSoonAsPossible(new Runnable() {
				public void run() {	delegee.filter(data, _offset, _subBlockLength); }
			});
		}
		workshop.waitForCompletion();
	}

}




