
package sam.io;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;


/** Generic type for I/O stream filters.
 ** <p>A stream filter is the association of mechanism for filtering an <CODE>InputStream</CODE>- and
 ** the corresponding de-filtering of an <CODE>OutputStream</CODE>.
 ** <p>Typical implementations are compressor like <CODE>Zip</CODE> or <CODE>Huffman</CODE>.
 **/
public abstract class StreamFilter {

	/** Filters (decompress) the archive input.
	 ** @param input the archive input.
	 ** @return a filtered stream from which plain data can be read.
	 **/
	abstract public InputStream filterInput(InputStream input) throws IOException;

	/** Filters (compress) the archive output.
	 ** @param output the archive output.
	 ** @return a filtered stream to which plain data can be written.
	 **/
	abstract public OutputStream filterOutput(OutputStream output) throws IOException;


	/** Finishes the archive output filtering.
	 ** @param output the filtered output.
	 ** <p>Typically a statement like <CODE>((GZipOutputStream) output).finish()</CODE>.
	 **/
	public void finishFilter(OutputStream output) throws IOException { }

}
