
package sam.toolkit;

import java.util.LinkedList;

/** Basic first-in first-out list implementation.
 ** This functionnality basically reduces to that of <CODE>java.util.LinkedList</CODE>,
 ** although this class allows for better semantics as it only defines two methods.
 ** Note that this class is thread-safe.
 **/
public class Fifo {

	private final LinkedList data;

	public Fifo() { data = new LinkedList(); }

 	/** Extracts the least recently <CODE>put</CODE> object from this <CODE>Fifo</CODE **/
	public synchronized Object get() { return data.removeFirst(); }

 	/** Adds one more object to this <CODE>Fifo</CODE **/
	public synchronized void put(Object object) { data.addLast(object); }

	public synchronized boolean isEmpty() { return data.size() == 0; }

}
