
package sam.toolkit;


/** This class is a <CODE>String</CODE> factory.
 ** Its purpose is to build strings of various length
 ** composed of the same repeated sequence in an efficient way.
 ** Note that all instances of this class should be <CODE>static</CODE>
 ** as there is no use for having multiple instances of the same factory
 ** (which would increase cache size).
 **/
public class StringFactory {

	/** Factory of blank strings (i.e. composed only of white spaces) **/
	public static final StringFactory SPACES = new StringFactory(" ");

	/** The root sequence to repeated **/
	private final String root;

	/** Builds a new <CODE>StringFactory</CODE>
	 ** @param root the root sequence to repeated.
	 **/
	public StringFactory(String root) { this.root = root; }

	/** Cache of the already calculated strings **/
	private String[] cache = new String[0];

	/** Builds the indentation prefix for specified length **/
	public String get(int length) {
		if (length <= 0)
			return "";
		if (length > cache.length)
			cache = rebuildCache(length);
		return cache[length - 1];
	}

	private String[] rebuildCache(int maximumLength) {
		String[] indentations = new String[maximumLength];
		StringBuffer buffer = new StringBuffer();
		for (int i=0; i< maximumLength; ++i) {
			buffer.append(root);
			indentations[i] = buffer.toString();
		}
		return indentations;
	}

}
