
package sam.ui.awt;

import sam.toolkit.Text;

import java.awt.*;


/** General-purpose information box.
 ** <p>For example:
 ** <pre><codeblock>
 ** EditBox box = new EditBox(this, "Title", "<Enter value here>");
 ** if (box.getChoice() == EditBox.OK)
 **    String text = box.getText();
 ** ...
 **
 ** </codeblock></pre>
 **/
public class EditBox extends Dialog {

	/** Pop-ups a new information box.
	 **	<p>The text can be multi-lined.
	 ** @param title the title of the dialog.
	 ** @param text the text to be displayed.
	 **/
	public EditBox(String title, String text) {
		this(title, text, (OK | CANCEL | EDITABLE) );
	}

	/** Pop-ups a new information box.
	 **	<p>The text can be multi-lined.
	 ** @param title the title of the dialog.
	 ** @param text the text to be displayed.
	 ** @param mode a combination of options for this dialog, e.g. <CODE>EditBox.OK | EditBox.CANCEL</COD>
	 **/
	public EditBox(String title, String text, int mode) {
		super(title, makeComponent(text), mode, 0);
	}

	/** Returns current text. **/
	public String getText() {
		return ((TextArea) getContent()).getText();
	}

	/** Gets the text component of this infobox. **/
	private static Component makeComponent(String text) {
		int lines = Text.countLines(text);
		TextArea textArea = new TextArea(text, lines + 1, 50, TextArea.SCROLLBARS_BOTH);
		textArea.setEditable(true);
		return textArea;
	}

}
