
package sam.ui.awt;

import java.awt.*;

/** Largely used: shortcut for adding several objects in a single line of code.
 ** <pre><codeblock>
 ** Container myContainer =  new Grid(2,3).append( new Label("foo") ).append().appendPanel( new Button("OK") );
 ** </codeblock></pre>
 **/
public class Grid extends Container {

	/** Builds a new grid
	 ** @param height the height of this grid (maximum number of vertical components that can be contained)
	 ** @param width the width of this grid (maximum number of horizontal components that can be contained)
	 **/
	public Grid(int height, int width) {
		setLayout( new GridLayout(height, width) );
	}

	/** Adds a new empty component.
	 ** This is equivalent to skipping one cell.
	 ** Component are added horizontally starting from the uppest raw.
	 **	@return <CODE>this</CODE> <CODE>Grid</CODE> object.
	 **/
	public Grid append() { return append( new Container() ); }

	/** Adds a new text component.
	 ** Component are added horizontally starting from the uppest raw.
	 **	@return <CODE>this</CODE> <CODE>Grid</CODE> object.
	 **/
	public Grid append(String text) { return append( new Label(text) );	}

	/** Adds a new component.
	 ** Component are added horizontally starting from the uppest raw.
	 **	@return <CODE>this</CODE> <CODE>Grid</CODE> object.
	 **/
	public Grid append(Component component) {
		super.add(component);
		return this;
	}

	/** Adds a new component inside a freshly built <CODE>Panel</CODE>.
	 ** This avoids the ugly "zooming" effect of adding components directly.
	 ** Component are added horizontally starting from the uppest raw.
	 **	@return <CODE>this</CODE> <CODE>Grid</CODE> object.
	 **/
	public Grid appendPanel(Component component) {
		Panel panel = new Panel();
		panel.add(component);
		return append(panel);
	}

}

