
package sam.ui.awt;

import java.awt.*;


/** Horizontal or vertical {@link sam.ui.awt.Grid} without a limit in the number of its components.
 **/
public class GrowableGrid extends Grid {

	private int height;
	private final boolean isVertical;
	private int count = 0;
	private Grid delegee;

	public GrowableGrid(int height, boolean isVertical) {
		super(1, 1);
		this.height = (height <= 0) ? 1 : height;
		this.isVertical = isVertical;
		this.delegee = new Grid(isVertical ? height : 1, isVertical ? 1 : height);
		super.append(delegee);
	}

	public synchronized Grid append(Component component) {
		if (count >= height)
			restore(2 * height);
		delegee.add(component, 0);
		return this;
	}

	public Component[] getComponents() { return delegee.getComponents(); }

	public void remove(Component component) { delegee.remove(component); }

	public void removeAll() { delegee.removeAll(); }

	private void restore(int height) {
		Grid newGrid = new Grid(isVertical ? height : 1, isVertical ? 1 : height );
		Component[] components = delegee.getComponents();
		for (int i=0; i<components.length; i++)
			newGrid.append(components[i]);
		delegee = newGrid;
		this.height = height;
		repaint();
	}

}

