
package sam.ui.awt;

import sam.core.Settings;
import sam.toolkit.Text;

import java.awt.*;


/** General-purpose information box.
 ** <p>For example:
 ** <pre><codeblock>
 ** if ( new InfoBox(this, "Title", "Question ?",
 **        InfoBox.YES | InfoBox.NO, InfoBox.NO_TIME_LIMIT).getChoice() == InfoBox.YES )
 ** ...
 **
 ** </codeblock></pre>
 **/
public class InfoBox extends Dialog {


	/** Pop-ups a new error message box
	 ** @param title the title of the dialog.
	 ** @param error the error to be reported
	 **/
	public InfoBox(String title, Throwable error) { this(title, Text.getMessage(error), OK, NO_TIME_LIMIT); }

	/** Pop-ups a new information box.
	 **	<p>The text can be multi-lined.
	 ** @param title the title of the dialog.
	 ** @param text the text to be displayed.
	 **/
	public InfoBox(String title, String text) {	this(title, text, OK); }

	/** Pop-ups a new information box.
	 **	<p>The text can be multi-lined.
	 ** @param title the title of the dialog.
	 ** @param text the text to be displayed.
	 ** @param mode a combination of options for this dialog, e.g. <CODE>InfoBox.OK | InfoBox.CANCEL</COD>
	 **/
	public InfoBox(String title, String text, int mode) {
		this(title, text, mode, Settings.Constants.UI.AWT.DIALOG_DEFAULT_LIFETIME);
	}

	/** Pop-ups a new information box.
	 **	<p>The text can be multi-lined.
	 ** @param title the title of the dialog.
	 ** @param text the text to be displayed.
	 ** @param mode a combination of options for this dialog, e.g. <CODE>InfoBox.OK | InfoBox.CANCEL</COD>
	 ** @param lifeTime time <b>in seconds</b> after which this box automatically gets dismissed.
	 **/
	public InfoBox(String title, String text, int mode, final long lifeTime) {
		super(title, makeComponent(text), mode, lifeTime);
	}

	/** Gets the text component of this infobox. **/
	private static Component makeComponent(String text) {
		int lines = Text.countLines(text);
		TextArea textArea = new TextArea(text, lines + 1, 50, TextArea.SCROLLBARS_NONE);
		textArea.setEditable(false);
		return textArea;
	}

}
