
package sam.ui.awt;

import java.awt.*;
import java.awt.event.*;


/** Simple integer input field **/
public class IntegerInputField extends Container {

	private final int min;

	private final int max;

	private final Scrollbar scrollbar;

	private final TextArea text;

	/** Builds a new integer input field.
	 ** @param value suggested value
	 ** @param min minimum allowed value
	 ** @param max maximum allowed value
	 **/
	public IntegerInputField(int value, int min, int max) {
		this.min = min;
		this.max = max;
		scrollbar = new Scrollbar(Scrollbar.HORIZONTAL, (max - min) / 50 + 5, value, min, max);
		text = new TextArea( ("" + value), 1, 20, TextArea.SCROLLBARS_NONE);

		setLayout( new BorderLayout() );
		add(scrollbar, BorderLayout.SOUTH);
		add(text, BorderLayout.CENTER);

		scrollbar.addAdjustmentListener(new AdjustmentListener() {
			public void adjustmentValueChanged(AdjustmentEvent e) { set( e.getValue() ); }
		} );
		text.addTextListener(new TextListener() {
			public void textValueChanged(TextEvent e) { set( text.getText().trim() ); }
		} );
	}

	private void set(int value) {	text.setText( "" + value );	}

	private void set(String value) {
		try {
			scrollbar.setValue( Integer.valueOf(value).intValue() );
		} catch (NumberFormatException e) {
		}
	}

	/** Gets the current value. **/
	public int getValue() {	return scrollbar.getValue(); }

}
