
package sam.ui.awt;

import sam.ui.*;
import sam.event.*;
import sam.core.Settings;

import java.awt.*;
import java.awt.event.*;


/** <a href=http://java.sun.com/docs/books/tutorial/ui/>AWT</a>-based implementation
 ** of a {@link sam.event.ProgressEventListener}
 **/
public class ProgressEventListener extends Frame implements sam.event.ProgressEventListener {

	/** Entry-related progress bar **/
	private ProgressBar entryBar = new ProgressBar();

	/** Global progress bar **/
	private ProgressBar globalBar = new ProgressBar();

	/** Entry-related text **/
	private Label entryText = new Label("");

	/** Global text **/
	private Label globalText = new Label("");

	/** Duration field **/
	private Label duration = new Label("");

	/** Compression ratio field **/
	private Label rate = new Label("");

	private Button closeButton;


	ProgressEventListener(LogInterface log) {
		super(Settings.Environment.PROGRAM_NAME);
		setLayout( new BorderLayout() );
		add(new Grid(3, 1).append( new Grid(1, 2).append(entryText).append(entryBar) )
				.append( new Grid(1, 2).append(globalText).append(globalBar) )
				.append( new Grid(1, 2).append(duration).append(rate) ),
				BorderLayout.CENTER);
		add(new Grid(1, 2).append( (closeButton = new Button("CANCEL", new ActionListener() {
				public void actionPerformed(ActionEvent e) { close(); }
				})) ).append( log.getButton(this) ), BorderLayout.SOUTH);
		addWindowListener( new WindowAdapter() {	public void windowClosing(WindowEvent e) {  close(); }  }  );
    pack();
		setSize(Settings.Constants.UI.AWT.PROGRESS_WINDOW_SIZE);
		setResizable(false);
		setLocation(100, 50);
		show();
	}

	protected void finalize() {
		hide();
		dispose();
		removeNotify();
	}


	protected void close() {
		if (UserInterface.get().running().yesNo("Exit", "Terminate application ?"))
			sam.toolkit.Process.exit();
	}

	/** This prints progress information in a not-so-beautiful way. **/
	public void progress(ProgressEvent event) {
		if ( event.global.isFinished() ) {
			entryText.setText(event.global.limitSizeString() + " done");
			entryBar.reset();
			globalText.setText(event.totalEntriesCount + " entries");
			closeButton.setLabel("Close");
			duration.setText("Total " + event.global.getTotalDurationString()  );
			repaint();
		} else {
			entryText.setText( event.currentEntryShortName(Settings.Constants.UI.AWT.ENTRY_NAME_LENGTH) );
			setBar(entryBar, event.entry);
		}
		globalText.setText( event.numberOfCurrentEntry + "/" + event.totalEntriesCount );
		setBar(globalBar, event.global);
		duration.setText( event.global.getRemainingDurationString() );
		rate.setText(" " + event.global.getRate() +
								 (Settings.Constants.UI.AWT.PRINT_COMPRESSION_RATIO ? (" " + event.global.getCompressionRatioString()) : "") );
	}

	private static void setBar(ProgressBar bar, ProgressEvent.Statistics statistics) {
		float plainRatio = (float) statistics.achieved / (float) statistics.limit;
		bar.setPlain(plainRatio);
		bar.setEncoded(statistics.encodingRatio * plainRatio);
		bar.setText(statistics.achievedPercentage() + "%");
	}


}

