
package sam.ui.awt;

import sam.ui.*;
import sam.toolkit.Text;
import sam.cipher.Specification;
import sam.cipher.Cipher;

import java.io.File;
import java.util.Stack;
import java.awt.List;
import java.awt.FileDialog;


/** <a href=http://java.sun.com/docs/books/tutorial/ui/>AWT</a>-based implementation
 ** of a {@link sam.ui.RulingInterface}
 **/
public class RulingInterface extends  sam.ui.RulingInterface {

	protected String enterString(String title, String suggestedValue) {
		EditBox box = new EditBox(title, suggestedValue, Dialog.INPUT);
		return (box.getChoice() == EditBox.OK) ? box.getText() : null;
	}

	protected int enterInteger(String title, int suggestedValue, int min, int max) {
		Dialog box = new Dialog(title, new IntegerInputField(suggestedValue, min, max), Dialog.INPUT);
		return (box.getChoice() == Dialog.OK) ? ((IntegerInputField) box.getContent()).getValue() : min;
	}

	protected File enterFileOrDirectory(String title, File suggestedValue, AccessMode mode) {
		String suggested = (suggestedValue == null) ? null : suggestedValue.getPath();
		String file = null;
		if ((mode == WRITABLE_DIRECTORY) || (mode == READABLE_DIRECTORY))
			file = enterString(title, null);
		else {
			FileDialog dialog = new FileDialog(new java.awt.Frame(title), title, (mode == WRITABLE_FILE) ? FileDialog.SAVE : FileDialog.LOAD );
			if (suggested != null)
				dialog.setFile(suggested);
			dialog.show();
			file = dialog.getFile();
		}
		return (file == null) ? null : new File(file);
	}

	protected String choose(String title, String[] values) {
		Dialog box = new Dialog(title, makeList(values), Dialog.INPUT);
		return (box.getChoice() == Dialog.OK) ? ((List) box.getContent()).getSelectedItem() : null;
	}

	protected Specification[] enterCiphers(String title, Specification[] ciphers, boolean canAddCiphers) {
		CipherInputField inputField = new CipherInputField(title, ciphers, canAddCiphers);
		return inputField.isCanceled() ? new Specification[0] : inputField.getSpecifications();
	}

	/** Builds a string choice list from a list of strings. **/
	private static List makeList(String[] values) {
		List list = new List(values.length, false);
		for (int i=0; i<values.length; )
			list.add( values[ i++ ] );
		return list;
	}


}
