
package sam.ui.awt;

import java.awt.*;
import java.awt.event.*;


/** Simple switchable button (e.g. "On/Off" button).
 **/
public abstract class SwitchButton extends java.awt.Button {

	/** Current state of the button **/
	private boolean isOn = true;

	/** String to be displayed whenever this button is set on **/
	private final String setOn;

	/** String to be displayed whenever this button is set off **/
	private final String setOff;

	/** Builds a new switch button.
	 ** The button is first set off by default.
	 ** @param setOn the string to be displayed whenever this button is set on
	 ** @param setOff the string to be displayed whenever this button is set off
	 **/
	public SwitchButton(String setOn, String setOff) {
		super(setOff);
		this.setOn = setOn;
		this.setOff = setOff;
		addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent event) { go(); }
		} );
	}

 	/** Called whenever this button is turned on
	 ** @return wether to change state again
	 **/
	abstract protected boolean on();

	/** Called whenever this button is turned off
	 ** @return wether to change state again
	 **/
	abstract protected boolean off();
		    
	void setState(boolean isOn) {
		this.isOn = isOn;
		setLabel(isOn ? setOff : setOn);
	}

	private void go() {
		if (isOn ? off() : on())  // OK this is a bit tricky ...
				setState( ! isOn);
	}
		
}


