
package sam.ui.awt;

import sam.core.Settings;
import sam.toolkit.Text;

import java.io.*;
import java.awt.*;


/** Text area writer.
 ** This is a simple bounded-recall <CODE>java.io.Writer</CODE> implementation
 ** that prints text to a graphical <CODE>java.awt.TextArea</CODE>
 **/
public class TextAreaWriter extends Writer {

	/** Graphical component do display text onto **/
	final TextArea display;

	/** Buffer to hold the log text **/
	private final StringBuffer buffer = new StringBuffer();

	/** Current log length **/
	private final int length;

	/** Builds a new <CODE>TextArea</CODE> </CODE>Writer</CODE>
	 ** @see Settings.Constants.UI.AWT#LOG_LENGTH
	 **/
	protected TextAreaWriter() { this(Settings.Constants.UI.AWT.LOG_LENGTH); }

	/** Builds a new <CODE>TextArea</CODE> </CODE>Writer</CODE>
	 ** @param length the maximum length of log material to be recalled.
	 **/
	protected TextAreaWriter(int length) {
		this.length = length;
		display = new TextArea("", 50, 80, TextArea.SCROLLBARS_VERTICAL_ONLY);
		display.setBackground( Color.lightGray );
		display.setEditable(false);
	}

	public synchronized void write(char[] c, int start, int size) {
		try {
			int kept = length - size;
			if (kept <= 0)
				buffer.setLength(0);
			else if (buffer.length() > kept) {
				int cut = buffer.length() - kept;
				while ( (cut > 0) && buffer.charAt(cut--) != '\n')
						;
				buffer.delete(0, cut + 1);
			}
			buffer.append(c, (kept >= 0) ? start : (start - kept), (kept >= 0) ? size : length);
			display.setText( buffer.toString() );
		} catch (Throwable t) {
			System.err.println("\nLOG FAILED: " + Text.getMessage(t) );
			t.printStackTrace();
		}
	}

	public void flush() { }

	public void close() {	flush(); }

}
