
package sam.ui.awt;

import sam.ui.*;
import sam.event.*;
import sam.toolkit.Text;

import java.io.*;


/** Pluggable <a href=http://java.sun.com/docs/books/tutorial/ui/>AWT</a>-based user interface
 **/
public class UserInterface extends sam.ui.UserInterface {

	static {
		if ( ! register(UserInterface.class, "awt") )  // Should never happend actually unless very strange conditions (low memory ...)
			System.err.println("Warning: could not register awt user interface");
	}

	/** Builds a new text user interface.
	 **/
	public UserInterface() { }

	public final sam.ui.RulingInterface ruling() { return new RulingInterface(); }

	private LogInterface log = null;

	public final RunningInterface running() {
		return new RunningInterface() {
			public FileEventListener getFileEventListener() {
				return new FileEventAdapter() {
					public void failed(FileEvent event)      { print("Failed entry", event.path() ); }
					public void missing(FileEvent event)     { print("Missing entry", event.path() ); }
					public void mismatching(FileEvent event) { print("Mismatching entry", event.path() ); }
				};
			}
			public sam.event.ProgressEventListener getProgressEventListener() {
				log();
				return new ProgressEventListener(log);
			}
			public void print(String title, String message) { new InfoBox(title, message); }
			public boolean yesNo(String title, String question) {
				return new InfoBox((title == null) ? "Question" : title, question, InfoBox.QUESTION, 0L).getChoice() == InfoBox.YES;
			}
		};
	}

	public final ErrorInterface error() {
		return new ErrorInterface() {
			public void unhandledParameter(String parameter) { error("Unhandled parameter", parameter); }
			public void incompatibleArchiveLayout(int version) { error("Incompatible archive version", "" + version); }
			public void handleException(String message, java.lang.Throwable e) {
				if ( running().yesNo("Abort ?", message + " since " + Text.getMessage(e) + "\nTerminate application ?") )
					error("Application terminated", (e == null) ? "" : e.getMessage() );
			}
			public void warn(String message) { new InfoBox("Warning", message); }
			protected void failure(String message) { error("application failed", message); }
			private void error(String title, String message) {
				running().print("Error: " + title, message);
				terminate("Fatal error.");
			}

		};
	}

	public sam.ui.LogInterface log() {
		if (log == null)
			log = new LogInterface();
		return log;
	}

	void terminate(String cause) throws Abort {
		throw new Abort(cause);
	}

}
