
package sam.ui.text;

import sam.ui.*;
import sam.toolkit.Text;
import sam.cipher.Specification;
import sam.cipher.Cipher;

import java.io.File;
import java.util.Stack;


/** Text-based implementation of a ruling interface
 **/
public class RulingInterface extends sam.ui.RulingInterface {

	/** The user interface to attach to **/
	private final UserInterface ui;

	/**
	 ** @param ui the user interface to attach to
	 **/
	RulingInterface(UserInterface ui) { this.ui = ui; }

	protected String enterString(String title, String suggestedValue) {
		ui.print("Enter " + title, null);
		return ui.readString();
	}

	protected int enterInteger(String title, int suggestedValue, int min, int max) {
		while (true) {
			ui.print("Enter " + title, null);
			String value = ui.readString();
			if ((value == null) || ("".equals( (value = value.trim()) ))) {  // Abort
				ui.println("Cancelled", "no value supplied for " + title + ", defaulting to " + suggestedValue);
				// Always returns the minimum
				return suggestedValue;
			}
			try {
				final int integer = Integer.valueOf(value).intValue();
				if ((integer >= min) && (integer <= max))
					return integer;
				else
					ui.println("Incorrect value", "integer " + integer + " is not in range [" + min + "," + max + "] !");
			} catch (NumberFormatException e) {
				ui.println("Incorrect value", "\"" + value + "\" is not a valid integer !");
			}
		}
	}

	protected File enterFileOrDirectory(String title, File suggestedValue, AccessMode mode) {
		ui.print("Enter file name for " + mode.description, null);
		String value = ui.readString();
		if ((value == null) || ("".equals( (value = value.trim()) ))) {  // Abort
			ui.println("Cancelled", "no " + mode.description + " supplied for " + title);
			return null;
		} else
			return new File(value);
	}

	protected String choose(String title, String[] values) {
		ui.print("Choose " + title + " among " + Text.toString(values), null);
		String value = ui.readString();
		if ((value == null) || ("".equals( (value = value.trim()) ))) {  // Abort
			ui.println("Cancelled", "using default choice");
			return null;
		} else
		return value;
	}

	protected Specification[] enterCiphers(String title, Specification[] ciphers, boolean canAddCiphers) {
		ui.println(title, "");
		if (canAddCiphers)
			ciphers = stackCiphers(ciphers);
		if (ciphers.length == 0)
			return new Specification[0];
		Specification[] specifications = new Specification[ciphers.length];
		for (int i=0; i<specifications.length; ++i)
			if (ciphers[i].isEmpty())	{
				ui.print("> Password for \"" + ciphers[i].name + "\" (" + (i + 1) + "): ");
				specifications[i] = new Specification(ciphers[i].name, ui.readString() );
			} else
				specifications[i] = ciphers[i];
		return specifications;
	}

	/** Chooses the stack of ciphers to be used
	 **/
	private Specification[] stackCiphers(Specification[] alreadyChosenCiphers) {
	 final Stack ciphers = (Stack) Text.addArray(new Stack(), alreadyChosenCiphers);
		ui.println("Available ciphers", Text.toString(Cipher.REGISTRY.getCiphers(), "", " ", ""));
		while (true) {
			String cipher = chooseString("another cipher to " + Text.toString( Specification
																																				 .getNames((Specification[])ciphers.toArray(new Specification[0])) )
																	 + " (RETURN terminates)", Cipher.REGISTRY.getCiphers(), "");
			if ((cipher == null) || ((cipher = cipher.trim()).equals("")))
				return (Specification[]) ciphers.toArray(new Specification[0]);
			else
				ciphers.push( new Specification(cipher, null) );
		}
	}

}
